﻿package {
	
	// Internal imports
	import flash.display.Bitmap;
	import flash.display.DisplayObject;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	// Library imports
	import nape.geom.Vec2;
	import nape.phys.Body;
	import nape.phys.BodyType;
	import nape.phys.Material;
	import nape.shape.Circle;
	import nape.shape.Polygon;
	import nape.space.Space;
	import nape.util.Debug;
	import nape.util.ShapeDebug;
	
	public class NapePhysicsIntro extends Sprite {
		
		/* To enable/disable our little debugging mode */
		private const DEBUG:Boolean = true;
		
		/* Reference to the Nape Physics space */
		private var space:Space;
		
		/* Reference to the floor physics body object */
		private var floorPhysicsBody:Body;
		
		/* Show shapes even if they don't have an attached DisplayObject (debug mode) */
		private var debug:ShapeDebug;
		
		/* Constructor */
		public function NapePhysicsIntro() {
			
			// Wait to be on stage before doing anything
			addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);

		}
		
		private function onAddedToStage(e:Event):void {
			
			removeEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
			
			// Create the nape space. This is where all the bodies in your physics
			// world will reside. To create this world you need to pass in a Vec2
			// object representing gravity. In this case, the first arg is the movement
			// on the x axis and the second arg is the movement on the y axis.
			space = new Space(new Vec2(0, 5000));
			
			// Create a debugging object that will draw shape outlines
			if (DEBUG) {
				debug = new ShapeDebug(stage.stageWidth, stage.stageHeight);
				addChild(debug.display);
			}
			
			// Create floor body (just below stage)
			floorPhysicsBody = new Body(BodyType.STATIC);
			var p:Polygon = new Polygon (
											Polygon.rect(
												0, 					// x position
												stage.stageHeight, 	// y position
												stage.stageWidth, 	// width
												100					// height
											)
										);
			floorPhysicsBody.shapes.add(p);
			floorPhysicsBody.space = space;
			
			stage.addEventListener(MouseEvent.MOUSE_DOWN, addSmiley);
			addEventListener(Event.ENTER_FRAME, loop);
			
		}		
		
		private function addSmiley(e:MouseEvent = null):void {
			
			var s:Sprite = new Smiley();
			addChild(s);
			
			// Create flag body and add it to the already-created space
			var smileyPhysicsBody:Body = new Body(BodyType.DYNAMIC, new Vec2(stage.mouseX, stage.mouseY));
			var material:Material = new Material(1.5);
			smileyPhysicsBody.shapes.add(new Circle(s.width >> 1, null, material));
			smileyPhysicsBody.space = space;
			
			// Assign a visual to the body
			smileyPhysicsBody.graphic = s;
			smileyPhysicsBody.graphicUpdate = updateGraphics;
			
		}
		
		private function updateGraphics(b:Body):void {
			// AS3 uses degrees to express rotation while nape uses radians. The modulo (%360) has been 
			// put in because AS3 does not like big numbers for rotation
			b.graphic.rotation = (b.rotation * 180 / Math.PI) % 360;
			b.graphic.x = b.position.x ;
			b.graphic.y = b.position.y ;
		}
		
		private function loop(e:Event):void {
			
			// Clear the debug drawings before we go to the next frame 
			if (DEBUG) debug.clear();
			
			// Go to next frame
			space.step(1 / stage.frameRate);
			
			// Draw the debug drawing and commit pending operations
			if (DEBUG) debug.draw(space);
			if (DEBUG) debug.flush();
			
			// Remove bodies and DisplayObjects that have moved offstage
			for(var i:uint = 0; i < space.bodies.length; i++) {
				
				// Get the current body in the loop
				var b:Body = space.bodies.at(i);
			
				// Do not delete the floor !
				if (b == floorPhysicsBody) break;
				
				// If the body is offstage, delete it along with the DisplayObject assigned 
				// to it
				if ( 
					 (b.position.x > stage.stageWidth + b.graphic.width) ||
					 (b.position.x < -b.graphic.width)
					) {
					b.graphic.parent.removeChild(b.graphic);
					space.bodies.remove(b);
					
				}
			}
			
		}
		
		
		
	}
	
}
